-module(case_clause_sender_pt).

-export([parse_transform/2,ref_append/1]).
-export([case_clause_sender_expr/1]).

ref_append(File) ->
	{ok, Forms} = epp:parse_file(File, [], []),
    Comments = erl_comment_scan:file(File),
    NForms = parse_transform(Forms,[]),
    lists:map(fun(Form) -> io:format("~p\n",[Form]) end, NForms),
    _FinalForms = erl_recomment:recomment_forms(NForms,Comments).
    %io:format("~s\n",[erl_prettypr:format(FinalForms)]).

parse_transform(Forms,_) ->
	NForms = [erl_syntax_lib:map(
		fun case_clause_sender_fun/1,
		Form) || Form <- Forms],
	%lists:map(fun(Form) -> io:format("~p\n",[Form]) end, NForms),
	NForms.

case_clause_sender_fun(T) ->
	case_clause_sender_expr(erl_syntax:revert(T)).


case_clause_sender_expr({'case',LINE,E,Clauses}) ->
	PatternsClauses = 
		[Pattern || {clause,_,Pattern,_,_} <- Clauses],
	{'case',LINE,E,change_clauses(Clauses,1,E,PatternsClauses,LINE)};

case_clause_sender_expr({'receive',LINE,Clauses}) ->
	PatternsClauses = 
		[Pattern || {clause,_,Pattern,_,_} <- Clauses],
	Nbody = [
	{match, LINE, {var, LINE,'MessageQueue'},{call,LINE,{remote,LINE,{atom,LINE,erlang},{atom,LINE,process_info}},[{call,LINE,{atom,LINE,self},[]},{atom,LINE,messages}]}},
	{op, LINE, '!',{atom,LINE,'case_tracer'},{tuple,LINE,[{var,LINE,'MessageQueue'},{call,LINE,{atom,LINE,self},[]}]}},
	{'receive',LINE,change_clauses2(Clauses,1,PatternsClauses,LINE)}],
		{block, LINE, Nbody};

case_clause_sender_expr({op, LINE,'!',To,Message})->
	Nbody = 
		[
		{op,LINE,'!',
			{atom,LINE,'case_tracer'}, 
			 {tuple,LINE,[
			 	{atom,LINE,'send_trace'},
			 	{tuple,LINE,[
			 		{call,LINE,{atom,LINE,self},[]},
				 	{string, LINE, lists:flatten(io_lib:format("~p ! ~p", [To, Message]))},
				 	{call,LINE,{remote,LINE,{atom,LINE,erlang},{atom,LINE,process_info}},[To,{atom,LINE,messages}]}
			 	]}
			 ]} },
			 {op,LINE,'!',To,{tuple,LINE,[{call,LINE,{remote,LINE,{atom,LINE,case_tracer},{atom,LINE,freshid}},[]},Message]}}

	 ],
	 %io:format("~nNBODY ~p",[Nbody]),
	 {block,LINE,Nbody};

case_clause_sender_expr(Other) ->
	Other.


change_clauses([],_,_,_,_) ->
	[];
change_clauses([{clause,LINE,Pattern,Guards,Body}|Clauses],Num,E,PatternsClauses,CaseLine) ->
	NClauses = change_clauses(Clauses,Num + 1,E,PatternsClauses,CaseLine),
	NBody = 
		[{op,LINE,'!',
			{atom,LINE,'case_tracer'},
			 {tuple,LINE,[
			 	{atom,LINE,'case_trace'},
			 	{tuple,LINE,[
				 	{integer,LINE,CaseLine},
				 	{integer,LINE,Num},
				 	{string,LINE,lists:flatten(io_lib:format("~p",[E]))},
				 	{string,LINE,lists:flatten(io_lib:format("~p",[Pattern]))},
				 	{string,LINE,lists:flatten(io_lib:format("~p",[PatternsClauses]))}
			 	]}
			 ]} }
		 | Body],
	[{clause,LINE,Pattern,Guards,NBody}|NClauses].

change_clauses2([],_,_,_)->
	[];
change_clauses2([{clause,LINE,Pattern,Guards,Body}|Clauses],Num,PatternsClauses,CaseLine) ->
	NClauses = change_clauses2(Clauses,Num + 1,PatternsClauses,CaseLine),
	NBody = 
		[{op,LINE,'!',
			{atom,LINE,'case_tracer'}, 
			 {tuple,LINE,[
			 	{atom,LINE,'receive_trace'},
			 	{tuple,LINE,[
				 	{integer,LINE,CaseLine},
				 	{integer,LINE,Num},
				 	{var,LINE,'MId'},
				 	{string,LINE,lists:flatten(io_lib:format("~p",[Pattern]))},
				 	{string,LINE,lists:flatten(io_lib:format("~p",[PatternsClauses]))}
			 	]}
			 ]} }
		 | Body],
		 NPattern = [{tuple,LINE,[{var,LINE,'MId'},hd(Pattern)]}],
	[{clause,LINE,NPattern,Guards,NBody}|NClauses].